@@ -1,5 +1,6 @@
 from ctypes.wintypes import *
 from typing import Any
+
 import struct
 import ctypes
 import locale
@@ -19,12 +20,17 @@ MEM_COMMIT = 0x00001000
 
 PAGE_EXECUTE_READ = 0x0020
 PAGE_EXECUTE_READWRITE = 0x0040
+PAGE_NOACCESS = 0x001
 PAGE_READONLY = 0x002
 PAGE_READWRITE = 0x004
+PAGE_WRITECOPY = 0x008
 
 MAX_PATH = 260
 
 # WinAPI обозначения типов
+ctypes.windll.kernel32.VirtualProtectEx.argtypes = [HANDLE, LPVOID, ctypes.c_size_t, DWORD, LPVOID]
+ctypes.windll.kernel32.VirtualProtectEx.restype = BOOL
+
 ctypes.windll.kernel32.OpenProcess.restype = HANDLE
 ctypes.windll.kernel32.VirtualAllocEx.restype = LPVOID
 
@@ -150,17 +156,22 @@ def mov_difference(offset: int) -> bytes:
 
     if offset < 0:
         hex_offset = hex(0xFFFF_FFFF_FFFF_FFFF + offset + 1)[10:]
+
         while len(hex_offset) != 8:
             hex_offset = '0' + hex_offset
+
         hex_offset = "".join(reversed([hex_offset[i: i + 2] for i in range(0, len(hex_offset), 2)]))
 
     else:
         hex_offset = hex(offset)[2:]
+
         while len(hex_offset) != 8:
             hex_offset = '0' + hex_offset
+
         hex_offset = "".join(reversed([hex_offset[i: i + 2] for i in range(0, len(hex_offset), 2)]))
 
     jump_bytecode += bytes.fromhex(hex_offset)
+
     return jump_bytecode
 
 
@@ -175,13 +186,17 @@ def list_processes_ids() -> list:
     """
 
     quantity = 32
+
     while True:
         process_ids = (DWORD * quantity)()
         quantity_buffer = ctypes.sizeof(process_ids)
+
         bytes_returned = DWORD()
         if ctypes.windll.Psapi.EnumProcesses(ctypes.byref(process_ids), quantity_buffer, ctypes.byref(bytes_returned)):
+
             if bytes_returned.value < quantity_buffer:
                 return list(set(process_ids))
+
             else:
                 quantity *= 2
 
@@ -198,21 +213,27 @@ def list_processes_names() -> list:
 
     quantity = 32
     processes_ids_list = []
+
     while True:
         process_ids = (DWORD * quantity)()
         quantity_buffer = ctypes.sizeof(process_ids)
+
         bytes_returned = DWORD()
         if ctypes.windll.Psapi.EnumProcesses(ctypes.byref(process_ids), quantity_buffer, ctypes.byref(bytes_returned)):
+
             if bytes_returned.value < quantity_buffer:
                 processes_ids_list = list(set(process_ids))
                 break
+
             else:
                 quantity *= 2
 
     processes_names = []
+
     for process_id in processes_ids_list:
         image_file_name = (ctypes.c_char * MAX_PATH)()
         process_handle = ctypes.windll.kernel32.OpenProcess(PROCESS_ALL_ACCESS, False, process_id)
+
         if ctypes.windll.psapi.GetProcessImageFileNameA(process_handle, image_file_name, MAX_PATH) > 0:
             filename = os.path.basename(image_file_name.value).decode()
             processes_names.append(filename)
@@ -232,25 +253,34 @@ def list_processes() -> list:
 
     quantity = 32
     processes_ids_list = []
+
     while True:
         process_ids = (DWORD * quantity)()
         quantity_buffer = ctypes.sizeof(process_ids)
         bytes_returned = DWORD()
+
         if ctypes.windll.Psapi.EnumProcesses(ctypes.byref(process_ids), quantity_buffer, ctypes.byref(bytes_returned)):
+
             if bytes_returned.value < quantity_buffer:
                 processes_ids_list = list(set(process_ids))
                 break
+
             else:
                 quantity *= 2
 
     processes = []
+
     for process_id in processes_ids_list:
+
         try:
+
             image_file_name = (ctypes.c_char * MAX_PATH)()
             process_handle = ctypes.windll.kernel32.OpenProcess(PROCESS_ALL_ACCESS, False, process_id)
+
             if ctypes.windll.psapi.GetProcessImageFileNameA(process_handle, image_file_name, MAX_PATH) > 0:
                 filename = os.path.basename(image_file_name.value).decode()
                 processes.append((filename, process_id))
+
         except Exception:
             pass
 
@@ -270,6 +300,7 @@ def is_64bit(process_handle: int) -> bool:
 
     wow_64 = ctypes.c_long()
     ctypes.windll.kernel32.IsWow64Process(process_handle, ctypes.byref(wow_64))
+
     return bool(wow_64)
 
 
@@ -280,9 +311,11 @@ def table_memory(process: object, address: int, rows: int, row_length: int = 24)
     """
 
     try:
-        for i in range(rows):
-            row_bytes = process.read_bytes(address + (i * row_length), row_length)
-            print(hex(address + (i * row_length))[2:len(hex(address + (i * row_length)))].upper() + ' | ' + heximate_bytes(row_bytes))
+
+        for row in range(rows):
+            row_bytes = process.read_bytes(address + (row * row_length), row_length)
+            print(hex(address + (row * row_length))[2:len(hex(address + (row * row_length)))].upper() + ' | ' + heximate_bytes(row_bytes))
+
     except Exception:
         MemoryAPIException("Невозможно построить таблицу по заданому адресу.")
 
@@ -298,18 +331,38 @@ def list_modules(process_handle: int) -> list:
     ['Discord.exe', 'ntdll.dll', 'KERNEL32.DLL', ..., 'wow64cpu.dll']
     """
 
-    hModules = (ctypes.c_void_p * 1024)()
-    process_module_success = ctypes.windll.psapi.EnumProcessModulesEx(process_handle, ctypes.byref(hModules), ctypes.sizeof(hModules), ctypes.byref(ctypes.c_ulong()), LIST_MODULES_ALL)
+    modules = (ctypes.c_void_p * 1024)()
+    process_module_success = ctypes.windll.psapi.EnumProcessModulesEx(process_handle, ctypes.byref(modules), ctypes.sizeof(modules), ctypes.byref(ctypes.c_ulong()), LIST_MODULES_ALL)
+
     if process_module_success:
         modules = []
-        hModules = iter(m for m in hModules if m)
-        for hModule in hModules:
+        modules = iter(module for module in modules if module)
+
+        for module in modules:
             module_info = MODULE(process_handle)
-            ctypes.windll.psapi.GetModuleInformation(process_handle, ctypes.c_void_p(hModule), ctypes.byref(module_info), ctypes.sizeof(module_info))
+            ctypes.windll.psapi.GetModuleInformation(process_handle, ctypes.c_void_p(module), ctypes.byref(module_info), ctypes.sizeof(module_info))
             modules.append(module_info.name)
+
         return modules
 
 
+def list_memory_sections(process_handle: int, end_address: int) -> list:
+    """Возвращает список (объекты MEMORY_BASIC_INFORMATION) со всеми секциями памяти процесса.
+
+    Return list (object MEMORY_BASIC_INFORMATION) of all memory section of process."""
+
+    return_list = []
+    check_region = 0
+
+    while check_region < end_address:
+        memory_region_information = virtual_query_ex(process_handle, check_region)
+        check_region = memory_region_information.BaseAddress + memory_region_information.RegionSize
+        return_list.append(memory_region_information)
+
+    else:
+        return return_list
+
+
 def virtual_alloc_ex(process_handle: int, bytes_length: int) -> int:
     """Выделяет указаное количество байт в памяти процесса.
 
@@ -324,6 +377,7 @@ def virtual_alloc_ex(process_handle: int, bytes_length: int) -> int:
     allocation_address = ctypes.windll.kernel32.VirtualAllocEx(process_handle, 0, bytes_length, MEM_COMMIT, PAGE_EXECUTE_READWRITE)
     if not allocation_address:
         raise MemoryAPIException("Handle процесса передан неверно, или размер буфера некорректен.")
+
     else:
         return allocation_address
 
@@ -342,9 +396,19 @@ def virtual_query_ex(process_handle: int, address: int) -> object:
 
     memory_information = MEMORY_BASIC_INFORMATION()
     ctypes.windll.kernel32.VirtualQueryEx(process_handle, ctypes.cast(address, ctypes.c_char_p), ctypes.byref(memory_information), ctypes.sizeof(memory_information))
+
     return memory_information
 
 
+def set_memory_section_protection(process_handle: int, address: int, page_size: int, desired_protection: int) -> None:
+    """Устанавливает защиту на участок памяти в данном процессе.
+
+    Set protection to memory region in given proccess."""
+
+    old_protection = ctypes.pointer(DWORD())
+    ctypes.windll.kernel32.VirtualProtectEx(process_handle, address, page_size, desired_protection, old_protection)
+
+
 class MemoryAPIException(Exception):
     """Базовый класс ошибок MemoryAPI.
 
@@ -420,6 +484,7 @@ class TrampolineHook:
         self.hook_bytecode = self.original_code + self.__backward__jump
         while len(self.hook_bytecode) != self.__alloc_buffer_size:
             self.hook_bytecode += b'\x00'
+
         else:
             self.__process.write_bytes(self.__instruction_address, self.hook_bytecode)
 
@@ -446,6 +511,7 @@ class TrampolineHook:
 
         if use_x64_registers and instruction_lenght < 12:
             raise MemoryAPIException("Длина инструкции должна быть не менее 12.")
+
         elif not use_x64_registers and instruction_lenght < 7:
             raise MemoryAPIException("Длина инструкции должна быть не менее 7.")
 
@@ -543,11 +609,13 @@ class Process:
 
             processes = list_processes()
             for process_name_iter, process_id_iter in processes:
+
                 if process_name_iter.lower() == introduced_process_name.lower():
                     self.pid = process_id_iter
                     self.name = process_name_iter
                     self.handle = ctypes.windll.kernel32.OpenProcess(PROCESS_ALL_ACCESS, False, process_id_iter)
                     break
+
             else:
                 raise MemoryAPIException("Процесс с данным именем не найден.")
 
@@ -558,11 +626,13 @@ class Process:
 
             processes = list_processes()
             for process_name_iter, process_id_iter in processes:
+
                 if process_id == process_id_iter:
                     self.pid = process_id_iter
                     self.name = process_name_iter
                     self.handle = ctypes.windll.kernel32.OpenProcess(PROCESS_ALL_ACCESS, False, process_id_iter)
                     break
+
             else:
                 raise MemoryAPIException("Процесс с данным ID не найден.")
 
@@ -583,11 +653,14 @@ class Process:
 
         hModules = (ctypes.c_void_p * 1024)()
         process_module_success = ctypes.windll.psapi.EnumProcessModulesEx(self.handle, ctypes.byref(hModules), ctypes.sizeof(hModules), ctypes.byref(ctypes.c_ulong()), LIST_MODULES_ALL)
+
         if process_module_success:
             hModules = iter(i for i in hModules if i)
+
             for hModule in hModules:
                 module_info = MODULE(self.handle)
                 ctypes.windll.psapi.GetModuleInformation(self.handle, ctypes.c_void_p(hModule), ctypes.byref(module_info), ctypes.sizeof(module_info))
+
                 if module_name.lower() == module_info.name.lower():
                     return module_info
 
@@ -616,37 +689,53 @@ class Process:
 
         if not pattern.count("?"):
             self.__bytes_pattern = bytes.fromhex("".join(self.__bytes_pattern))
+
         else:
             temp_pattern = b""
+
             for byte in self.__bytes_pattern:
+
                 if byte == "??":
                     temp_pattern += b'.'
+
                 else:
                     temp_pattern += bytes.fromhex(byte)
+
             self.__bytes_pattern = temp_pattern
 
         self.__scan_sections = []
         allowed_protections = [PAGE_EXECUTE_READ, PAGE_EXECUTE_READWRITE, PAGE_READWRITE, PAGE_READONLY]
+
         while scan_region < end_address:
             memory_region_information = virtual_query_ex(self.handle, scan_region)
             scan_region = memory_region_information.BaseAddress + memory_region_information.RegionSize
+
             if not (memory_region_information.State != MEM_COMMIT or memory_region_information.Protect not in allowed_protections):
                 self.__scan_sections.append((memory_region_information.BaseAddress, memory_region_information.RegionSize))
 
         addresses = []
         for section in self.__scan_sections:
+
             try:
                 self.__page = self.read_bytes(section[0], section[1])
+
                 if not return_first_found:
+
                     for match in re.finditer(self.__bytes_pattern, self.__page, re.DOTALL):
+
                         addresses.append(section[0] + match.span()[0])
                 else:
+
                     for match in re.finditer(self.__bytes_pattern, self.__page, re.DOTALL):
+
                         return section[0] + match.span()[0]
+
             except Exception:
                 pass
+
         if return_first_found:
             return None
+
         else:
             return addresses
 
@@ -673,37 +762,51 @@ class Process:
 
         if not pattern.count("?"):
             self.__bytes_pattern = bytes.fromhex("".join(self.__bytes_pattern))
+
         else:
             temp_pattern = b""
+
             for byte in self.__bytes_pattern:
+
                 if byte == "??":
                     temp_pattern += b'.'
+
                 else:
                     temp_pattern += b'\\' + 'x'.encode() + byte.encode()
+
             self.__bytes_pattern = temp_pattern
 
         self.__scan_sections = []
         allowed_protections = [PAGE_EXECUTE_READ, PAGE_EXECUTE_READWRITE, PAGE_READWRITE, PAGE_READONLY]
+
         while scan_region < end_address:
             memory_region_information = virtual_query_ex(self.handle, scan_region)
             scan_region = memory_region_information.BaseAddress + memory_region_information.RegionSize
+
             if not (memory_region_information.State != MEM_COMMIT or memory_region_information.Protect not in allowed_protections):
                 self.__scan_sections.append((memory_region_information.BaseAddress, memory_region_information.RegionSize))
 
         addresses = []
         for section in self.__scan_sections:
+
             try:
                 self.__page = self.read_bytes(section[0], section[1])
+
                 if not return_first_found:
+
                     for match in re.finditer(self.__bytes_pattern, self.__page, re.DOTALL):
                         addresses.append(section[0] + match.span()[0])
+
                 else:
                     for match in re.finditer(self.__bytes_pattern, self.__page, re.DOTALL):
                         return section[0] + match.span()[0]
+
             except Exception:
                 pass
+
         if return_first_found:
             return None
+
         else:
             return addresses
 
@@ -722,6 +825,7 @@ class Process:
         buffer = ctypes.create_string_buffer(length)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), length, ctypes.byref(bytes_read))
+
         return buffer.raw
 
     def read_short(self, address: int) -> int:
@@ -738,6 +842,7 @@ class Process:
         buffer = ctypes.create_string_buffer(2)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 2, ctypes.byref(bytes_read))
+
         return struct.unpack('<h', buffer.raw)[0]
 
     def read_ushort(self, address: int) -> int:
@@ -754,6 +859,7 @@ class Process:
         buffer = ctypes.create_string_buffer(2)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 2, ctypes.byref(bytes_read))
+
         return struct.unpack('<H', buffer.raw)[0]
 
     def read_int(self, address: int) -> int:
@@ -770,6 +876,7 @@ class Process:
         buffer = ctypes.create_string_buffer(4)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 4, ctypes.byref(bytes_read))
+
         return struct.unpack('<i', buffer.raw)[0]
 
     def read_uint(self, address: int) -> int:
@@ -786,6 +893,7 @@ class Process:
         buffer = ctypes.create_string_buffer(4)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 4, ctypes.byref(bytes_read))
+
         return struct.unpack('<I', buffer.raw)[0]
 
     def read_long(self, address: int) -> int:
@@ -802,6 +910,7 @@ class Process:
         buffer = ctypes.create_string_buffer(4)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 4, ctypes.byref(bytes_read))
+
         return struct.unpack('<l', buffer.raw)[0]
 
     def read_ulong(self, address: int) -> int:
@@ -818,6 +927,7 @@ class Process:
         buffer = ctypes.create_string_buffer(4)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 4, ctypes.byref(bytes_read))
+
         return struct.unpack('<L', buffer.raw)[0]
 
     def read_longlong(self, address: int) -> int:
@@ -834,6 +944,7 @@ class Process:
         buffer = ctypes.create_string_buffer(8)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 8, ctypes.byref(bytes_read))
+
         return struct.unpack('<q', buffer.raw)[0]
 
     def read_ulonglong(self, address: int) -> int:
@@ -850,6 +961,7 @@ class Process:
         buffer = ctypes.create_string_buffer(8)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 8, ctypes.byref(bytes_read))
+
         return struct.unpack('<Q', buffer.raw)[0]
 
     def read_float(self, address: int) -> float:
@@ -865,6 +977,7 @@ class Process:
         buffer = ctypes.create_string_buffer(4)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 4, ctypes.byref(bytes_read))
+
         return struct.unpack('<f', buffer.raw)[0]
 
     def read_double(self, address: int) -> float:
@@ -881,6 +994,7 @@ class Process:
         buffer = ctypes.create_string_buffer(8)
         bytes_read = ctypes.c_size_t()
         ctypes.windll.kernel32.ReadProcessMemory(self.handle, ctypes.c_void_p(address), ctypes.byref(buffer), 8, ctypes.byref(bytes_read))
+
         return struct.unpack('<d', buffer.raw)[0]
 
     # Запись памяти
